@@ -22,7 +22,7 @@ from .db_mapping_query_mixin import DatabaseMappingQueryMixin
 from .db_mapping_check_mixin import DatabaseMappingCheckMixin
 from .db_mapping_add_mixin import DatabaseMappingAddMixin
 from .db_mapping_update_mixin import DatabaseMappingUpdateMixin
-from .diff_db_mapping_remove_mixin import DiffDatabaseMappingRemoveMixin
+from .db_mapping_remove_mixin import DatabaseMappingRemoveMixin
 from .diff_db_mapping_commit_mixin import DiffDatabaseMappingCommitMixin
 from .diff_db_mapping_base import DiffDatabaseMappingBase
 from .filters.tools import apply_filter_stack, load_filters
@@ -34,7 +34,7 @@ class DiffDatabaseMapping(
     DatabaseMappingCheckMixin,
     DatabaseMappingAddMixin,
     DatabaseMappingUpdateMixin,
-    DiffDatabaseMappingRemoveMixin,
+    DatabaseMappingRemoveMixin,
     DiffDatabaseMappingCommitMixin,
     DiffDatabaseMappingBase,
 ):
@@ -56,12 +56,13 @@ class DiffDatabaseMapping(
             stack = load_filters(self._filter_configs)
             apply_filter_stack(self, stack)
 
-    def _add_items(self, tablename, *items, return_items=False):
-        items_to_add, ids = self._items_and_ids(tablename, *items)
-        for tablename_ in self._do_add_items(tablename, *items_to_add):
+    def _add_items(self, tablename, *items):
+        self._add_commit_id_and_ids(tablename, *items)
+        ids = {x["id"] for x in items}
+        for tablename_ in self._do_add_items(tablename, *items):
             self.added_item_id[tablename_].update(ids)
             self._clear_subqueries(tablename_)
-        return items_to_add if return_items else ids
+        return ids
 
     def _readd_items(self, tablename, *items):
         ids = set(x["id"] for x in items)
@@ -91,28 +92,20 @@ class DiffDatabaseMapping(
                 dirty_ids.add(id_)
         return items_for_update, items_for_insert, dirty_ids, updated_ids
 
-    def _update_items(self, tablename, *items):
-        """Update items without checking integrity."""
-        real_tablename = {
-            "object_class": "entity_class",
-            "relationship_class": "entity_class",
-            "object": "entity",
-            "relationship": "entity",
-        }.get(tablename, tablename)
-        items = self._items_with_type_id(tablename, *items)
+    def _do_update_items(self, tablename, *items):
         try:
             items_for_update, items_for_insert, dirty_ids, updated_ids = self._get_items_for_update_and_insert(
-                real_tablename, items
+                tablename, items
             )
-            self._do_update_items(real_tablename, items_for_update, items_for_insert)
-            self._mark_as_dirty(real_tablename, dirty_ids)
-            self.updated_item_id[real_tablename].update(dirty_ids)
+            self._update_and_insert_items(tablename, items_for_update, items_for_insert)
+            self._mark_as_dirty(tablename, dirty_ids)
+            self.updated_item_id[tablename].update(dirty_ids)
             return updated_ids
         except DBAPIError as e:
             msg = f"DBAPIError while updating {tablename} items: {e.orig.args}"
             raise SpineDBAPIError(msg)
 
-    def _do_update_items(self, tablename, items_for_update, items_for_insert):
+    def _update_and_insert_items(self, tablename, items_for_update, items_for_insert):
         diff_table = self._diff_table(tablename)
         if items_for_update:
             upd = diff_table.update()
@@ -123,18 +116,6 @@ class DiffDatabaseMapping(
         ins = diff_table.insert()
         self._checked_execute(ins, items_for_insert)
 
-    # pylint: disable=arguments-differ
-    def update_wide_relationships(self, *items, check=True, strict=False, return_items=False, cache=None):
-        """Update relationships."""
-        if check:
-            checked_items, intgr_error_log = self.check_wide_relationships_for_update(
-                *items, strict=strict, cache=cache
-            )
-        else:
-            checked_items, intgr_error_log = items, []
-        updated_ids = self._update_wide_relationships(*checked_items)
-        return checked_items if return_items else updated_ids, intgr_error_log
-
     def _update_wide_relationships(self, *items):
         """Update relationships without checking integrity."""
         items = self._items_with_type_id("relationship", *items)
@@ -163,13 +144,34 @@ class DiffDatabaseMapping(
                 dirty_rel_ent_ids,
                 updated_rel_ent_ids,
             ) = self._get_items_for_update_and_insert("relationship_entity", rel_ent_items)
-            self._do_update_items("entity", ents_for_update, ents_for_insert)
+            self._update_and_insert_items("entity", ents_for_update, ents_for_insert)
             self._mark_as_dirty("entity", dirty_ent_ids)
             self.updated_item_id["entity"].update(dirty_ent_ids)
-            self._do_update_items("relationship_entity", rel_ents_for_update, rel_ents_for_insert)
+            self._update_and_insert_items("relationship_entity", rel_ents_for_update, rel_ents_for_insert)
             self._mark_as_dirty("relationship_entity", dirty_rel_ent_ids)
             self.updated_item_id["relationship_entity"].update(dirty_rel_ent_ids)
             return updated_ent_ids.union(updated_rel_ent_ids)
         except DBAPIError as e:
             msg = "DBAPIError while updating relationships: {}".format(e.orig.args)
             raise SpineDBAPIError(msg)
+
+    def remove_items(self, **kwargs):
+        """Removes items by id, *not in cascade*.
+
+        Args:
+            **kwargs: keyword is table name, argument is list of ids to remove
+        """
+        for tablename, ids in kwargs.items():
+            table_id = self.table_ids.get(tablename, "id")
+            diff_table = self._diff_table(tablename)
+            delete = diff_table.delete().where(self.in_(getattr(diff_table.c, table_id), ids))
+            try:
+                self.connection.execute(delete)
+            except DBAPIError as e:
+                msg = f"DBAPIError while removing {tablename} items: {e.orig.args}"
+                raise SpineDBAPIError(msg)
+        for tablename, ids in kwargs.items():
+            self.added_item_id[tablename].difference_update(ids)
+            self.updated_item_id[tablename].difference_update(ids)
+            self.removed_item_id[tablename].update(ids)
+            self._mark_as_dirty(tablename, ids)
