@@ -17,8 +17,7 @@ Contains export mappings for database items such as entities, entity classes and
 
 from dataclasses import dataclass
 from itertools import cycle, dropwhile, islice
-import re
-from sqlalchemy import and_
+from sqlalchemy import and_, or_
 from sqlalchemy.sql.expression import literal
 from ..parameter_value import (
     from_database_to_single_value,
@@ -61,25 +60,18 @@ class ExportMapping(Mapping):
 
     _TITLE_SEP = ","
 
-    def __init__(self, position, value=None, header="", filter_re="", group_fn=None):
+    def __init__(self, position, value=None, header="", filter_re=""):
         """
         Args:
             position (int or Position): column index or Position
             value (Any, optional): A fixed value
-            header (str, optional); A string column header that's yielded as 'first row', if not empty.
+            header (str); A string column header that's yielded as 'first row', if not empty.
                 The default is an empty string (so it's not yielded).
-            filter_re (str, optional): A regular expression to filter the mapped values by
-            group_fn (str, Optional): Only for topmost mappings. The name of one of our supported group functions,
-                for aggregating values over repeated 'headers' (in tables with hidden elements).
-                If None (the default), then no such aggregation is performed and 'headers' are just repeated as needed.
+            filter_re (str): A regular expression to filter the mapped values by
         """
-        super().__init__(position, value=value)
-        self._filter_re = ""
-        self._group_fn = None
+        super().__init__(position, value, filter_re)
         self._ignorable = False
         self.header = header
-        self.filter_re = filter_re
-        self.group_fn = group_fn
         self._convert_data = None
 
     def __eq__(self, other):
@@ -87,20 +79,7 @@ class ExportMapping(Mapping):
             return NotImplemented
         if not super().__eq__(other):
             return False
-        return (
-            self._filter_re == other._filter_re
-            and self._group_fn == other._group_fn
-            and self._ignorable == other._ignorable
-            and self.header == other.header
-        )
-
-    @property
-    def filter_re(self):
-        return self._filter_re
-
-    @filter_re.setter
-    def filter_re(self, filter_re):
-        self._filter_re = filter_re
+        return self._ignorable == other._ignorable and self.header == other.header
 
     def check_validity(self):
         """Checks if mapping is valid.
@@ -175,30 +154,25 @@ class ExportMapping(Mapping):
             mapping_dict["ignorable"] = True
         if self.header:
             mapping_dict["header"] = self.header
-        if self.filter_re:
-            mapping_dict["filter_re"] = self.filter_re
-        if self.group_fn and self.group_fn != NoGroup.NAME:
-            mapping_dict["group_fn"] = self.group_fn
         return mapping_dict
 
     @classmethod
-    def reconstruct(cls, position, ignorable, mapping_dict):
+    def reconstruct(cls, position, value, header, filter_re, ignorable, mapping_dict):
         """
         Reconstructs mapping.
 
         Args:
             position (int or Position, optional): mapping's position
+            value (Any): fixed value
+            header (str, optional): column header
+            filter_re (str): filter regular expression
             ignorable (bool): ignorable flag
-            mapping_dict (dict): serialized mapping
+            mapping_dict (dict): mapping dict
 
         Returns:
             Mapping: reconstructed mapping
         """
-        value = mapping_dict.get("value")
-        header = mapping_dict.get("header", "")
-        filter_re = mapping_dict.get("filter_re", "")
-        group_fn = mapping_dict.get("group_fn")
-        mapping = cls(position, value=value, header=header, filter_re=filter_re, group_fn=group_fn)
+        mapping = cls(position, value, header, filter_re)
         mapping.set_ignorable(ignorable)
         return mapping
 
@@ -231,9 +205,10 @@ class ExportMapping(Mapping):
         return query
 
     def filter_query_by_title(self, query, title_state):
-        """Filters the query using pertinent information from the given title state.
-        Typically, title filters can be applied by calling ``filter()`` on the query directly (see ``_build_query()``).
-        However, subclasses may need something more specific, for example, to filter parameter values by type.
+        """Filters the query according to the given title state.
+        Note that ``_build_query()`` does some default filtering on the title state after calling this method.
+        Therefore, if a subclass reimplements this method, it needs to delete the consumed keys from ``title_state``
+        so they aren't consumed again by ``_build_query()``.
 
         The base class implementations just returns the unaltered query.
 
@@ -264,17 +239,78 @@ class ExportMapping(Mapping):
         # Apply filters
         for m in mappings:
             qry = m.filter_query(db_map, qry)
-        # Create a subquery to apply title filters
-        sq = qry.subquery()
-        qry = db_map.query(sq)
         # Apply special title filters (first, so we clean up the state)
         for m in mappings:
             qry = m.filter_query_by_title(qry, title_state)
         # Apply standard title filters
-        for key, value in title_state.items():
-            qry = qry.filter(getattr(sq.c, key) == value)
+        if not title_state:
+            return qry
+        # Use a _FilteredQuery, since building a subquery to query it again leads to parser stack overflow
+        return _FilteredQuery(
+            qry, lambda db_row: all(getattr(db_row, key) == value for key, value in title_state.items())
+        )
+
+    def _build_title_query(self, db_map):
+        """Builds and returns the query to get titles for this mapping hierarchy.
+
+        Args:
+            db_map (DatabaseMappingBase): database mapping
+
+        Returns:
+            Alias: title query
+        """
+        mappings = self.flatten()
+        for i in range(len(mappings)):
+            if mappings[-1].position == Position.table_name:
+                break
+            mappings.pop(-1)
+        # Start with empty query
+        qry = db_map.query(literal(None))
+        # Add columns
+        for m in mappings:
+            qry = m.add_query_columns(db_map, qry)
+        # Apply filters
+        for m in mappings:
+            qry = m.filter_query(db_map, qry)
         return qry
 
+    def _build_header_query(self, db_map, title_state, buddies):
+        """Builds the header query for this mapping hierarchy.
+
+        Args:
+            db_map (DatabaseMappingBase): database mapping
+            title_state (dict): title state
+            buddies (list of tuple): pairs of buddy mappings
+
+        Returns:
+            Alias: header query
+        """
+        mappings = self.flatten()
+        flat_buddies = [b for pair in buddies for b in pair]
+        for _ in range(len(mappings)):
+            m = mappings[-1]
+            if m.position == Position.header or m.position == Position.table_name or m in flat_buddies:
+                break
+            mappings.pop(-1)
+        # Start with empty query
+        qry = db_map.query(literal(None))
+        # Add columns
+        for m in mappings:
+            qry = m.add_query_columns(db_map, qry)
+        # Apply filters
+        for m in mappings:
+            qry = m.filter_query(db_map, qry)
+        # Apply special title filters (first, so we clean up the state)
+        for m in mappings:
+            qry = m.filter_query_by_title(qry, title_state)
+        # Apply standard title filters
+        if not title_state:
+            return qry
+        # Use a _FilteredQuery, since building a subquery to query it again leads to parser stack overflow
+        return _FilteredQuery(
+            qry, lambda db_row: all(getattr(db_row, key) == value for key, value in title_state.items())
+        )
+
     @staticmethod
     def name_field():
         """Returns the 'name' field associated to this mapping within the query.
@@ -333,18 +369,17 @@ class ExportMapping(Mapping):
             generator(any)
         """
         data_iterator = self._expand_data(data)
-        if self._filter_re:
-            data_iterator = (x for x in data_iterator if re.search(self._filter_re, str(x)))
+        if self._filter_re is not None:
+            data_iterator = (x for x in data_iterator if self._filter_re.search(str(x)))
         if self._convert_data is not None:
             data_iterator = (self._convert_data(x) for x in data_iterator)
         return data_iterator
 
-    def _get_rows(self, db_row, limit=None):
+    def _get_rows(self, db_row):
         """Yields rows issued by this mapping for given database row.
 
         Args:
             db_row (KeyedTuple)
-            limit (int, optional): yield only this many items
 
         Returns:
             generator(dict)
@@ -356,44 +391,40 @@ class ExportMapping(Mapping):
         if data is None and not self._ignorable:
             return ()
         data_iterator = self._get_data_iterator(data)
-        if limit is not None:
-            data_iterator = islice(data_iterator, limit)
         for data in data_iterator:
             yield {self.position: data}
 
-    def get_rows_recursive(self, db_row, limit=None):
+    def get_rows_recursive(self, db_row):
         """Takes a database row and yields rows issued by this mapping and its children combined.
 
         Args:
             db_row (KeyedTuple)
-            limit (int, optional): yield only this many items
 
         Returns:
             generator(dict)
         """
         if self.child is None:
-            yield from self._get_rows(db_row, limit=limit)
+            yield from self._get_rows(db_row)
             return
-        for row in self._get_rows(db_row, limit=limit):
-            for child_row in self.child.get_rows_recursive(db_row, limit=limit):
+        for row in self._get_rows(db_row):
+            for child_row in self.child.get_rows_recursive(db_row):
                 row = row.copy()
                 row.update(child_row)
                 yield row
 
-    def rows(self, db_map, title_state, limit=None):
+    def rows(self, db_map, title_state):
         """Yields rows issued by this mapping and its children combined.
 
         Args:
             db_map (DatabaseMappingBase)
             title_state (dict)
-            limit (int, optional): yield only this many items
 
         Returns:
             generator(dict)
         """
         qry = self._build_query(db_map, title_state)
         for db_row in qry.yield_per(1000):
-            yield from self.get_rows_recursive(db_row, limit=limit)
+            yield from self.get_rows_recursive(db_row)
 
     def has_titles(self):
         """Returns True if this mapping or one of its children generates titles.
@@ -442,7 +473,7 @@ class ExportMapping(Mapping):
         data_iterator = self._get_data_iterator(data)
         if limit is not None:
             data_iterator = islice(data_iterator, limit)
-        for data in self._get_data_iterator(data):
+        for data in data_iterator:
             if data is None:
                 data = ""
             yield data, title_state
@@ -476,7 +507,7 @@ class ExportMapping(Mapping):
         Yields:
             tuple(str,dict): title, and associated title state dictionary
         """
-        qry = self._build_query(db_map, dict())
+        qry = self._build_title_query(db_map)
         for db_row in qry.yield_per(1000):
             yield from self.get_titles_recursive(db_row, limit=limit)
 
@@ -543,7 +574,7 @@ class ExportMapping(Mapping):
         Returns
             dict: a mapping from column index to string header
         """
-        qry = self._build_query(db_map, title_state)
+        qry = self._build_header_query(db_map, title_state, buddies)
         first_row = qry.first()
         return self.make_header_recursive(first_row, title_state, buddies)
 
@@ -574,7 +605,7 @@ class FixedValueMapping(ExportMapping):
 
     MAP_TYPE = "FixedValue"
 
-    def __init__(self, position, value, header="", filter_re="", group_fn=None):
+    def __init__(self, position, value, header="", filter_re=""):
         """
         Args:
             position (int or Position, optional): mapping's position
@@ -582,11 +613,8 @@ class FixedValueMapping(ExportMapping):
             header (str, optional); A string column header that's yielt as 'first row', if not empty.
                 The default is an empty string (so it's not yielt).
             filter_re (str, optional): A regular expression to filter the mapped values by
-            group_fn (str, Optional): Only for topmost mappings. The name of one of our supported group functions,
-                for aggregating values over repeated 'headers' (in tables with hidden elements).
-                If None (the default), then no such aggregation is performed and 'headers' are just repeated as needed.
         """
-        super().__init__(position, value, header, filter_re, group_fn)
+        super().__init__(position, value, header, filter_re)
 
     @staticmethod
     def name_field():
@@ -729,13 +757,73 @@ class RelationshipClassMapping(ExportMapping):
         # Use the class name here, for the sake of the standard excel export
         return "relationship_class_name"
 
+    def query_parents(self, what):
+        if what != "dimension":
+            return super().query_parents(what)
+        return -1
+
     def _title_state(self, db_row):
         state = super()._title_state(db_row)
         state["object_class_id_list"] = getattr(db_row, "object_class_id_list")
         return state
 
-    def index(self):
-        return -1
+
+class RelationshipClassObjectHighlightingMapping(RelationshipClassMapping):
+    """Maps relationships classes.
+
+    Adds object class dimension chosen by highlight_dimension to the query.
+
+    Can be used as the topmost mapping.
+    """
+
+    MAP_TYPE = "RelationshipClassObjectHighlightingMapping"
+
+    def __init__(self, position, value=None, header="", filter_re="", highlight_dimension=0):
+        super().__init__(position, value, header, filter_re)
+        self._highlight_dimension = highlight_dimension
+
+    @property
+    def highlight_dimension(self):
+        return self._highlight_dimension
+
+    @highlight_dimension.setter
+    def highlight_dimension(self, dimension):
+        self._highlight_dimension = dimension
+
+    def add_query_columns(self, db_map, query):
+        query = super().add_query_columns(db_map, query)
+        return query.add_columns(db_map.object_class_sq.c.id.label("object_class_id"))
+
+    def filter_query(self, db_map, query):
+        highlighted_object_class_qry = db_map.query(db_map.relationship_class_sq).filter(
+            db_map.relationship_class_sq.c.dimension == self._highlight_dimension
+        )
+        conditions = (
+            and_(db_map.wide_relationship_class_sq.c.id == x.id, db_map.object_class_sq.c.id == x.object_class_id)
+            for x in highlighted_object_class_qry
+        )
+        return query.filter(or_(*conditions))
+
+    @staticmethod
+    def id_field():
+        return "relationship_class_id"
+
+    def query_parents(self, what):
+        if what != "highlight_dimension":
+            return super().query_parents(what)
+        return self._highlight_dimension
+
+    def to_dict(self):
+        mapping_dict = super().to_dict()
+        mapping_dict["highlight_dimension"] = self._highlight_dimension
+        return mapping_dict
+
+    @classmethod
+    def reconstruct(cls, position, value, header, filter_re, ignorable, mapping_dict):
+        highlight_dimension = mapping_dict["highlight_dimension"]
+        mapping = cls(position, value, header, filter_re, highlight_dimension)
+        mapping.set_ignorable(ignorable)
+        return mapping
 
 
 class RelationshipClassObjectClassMapping(ExportMapping):
@@ -745,6 +833,7 @@ class RelationshipClassObjectClassMapping(ExportMapping):
     """
 
     MAP_TYPE = "RelationshipClassObjectClass"
+    _cached_dimension = None
 
     @staticmethod
     def name_field():
@@ -756,14 +845,17 @@ class RelationshipClassObjectClassMapping(ExportMapping):
 
     def _data(self, db_row):
         data = super()._data(db_row).split(",")
-        index = self.index()
+        if self._cached_dimension is None:
+            self._cached_dimension = self.query_parents("dimension")
         try:
-            return data[index]
+            return data[self._cached_dimension]
         except IndexError:
             return ""
 
-    def index(self):
-        return self.parent.index() + 1
+    def query_parents(self, what):
+        if what != "dimension":
+            return super().query_parents(what)
+        return self.parent.query_parents(what) + 1
 
     @staticmethod
     def is_buddy(parent):
@@ -800,6 +892,11 @@ class RelationshipMapping(ExportMapping):
     def id_field():
         return "relationship_id"
 
+    def query_parents(self, what):
+        if what != "dimension":
+            return super().query_parents(what)
+        return -1
+
     def _title_state(self, db_row):
         state = super()._title_state(db_row)
         state["object_id_list"] = getattr(db_row, "object_id_list")
@@ -809,8 +906,35 @@ class RelationshipMapping(ExportMapping):
     def is_buddy(parent):
         return isinstance(parent, RelationshipClassMapping)
 
-    def index(self):
-        return -1
+
+class RelationshipObjectHighlightingMapping(RelationshipMapping):
+    """Maps relationships.
+
+    Adds object dimension chosen by highlight_dimension in relationship class mapping to the query.
+
+    Cannot be used as the topmost mapping;
+    one of the parents must be :class:`RelationshipClassObjectHighlightingMapping`.
+    """
+
+    MAP_TYPE = "RelationshipObjectHighlightingMapping"
+
+    def add_query_columns(self, db_map, query):
+        query = super().add_query_columns(db_map, query)
+        return query.add_columns(db_map.object_sq.c.id.label("object_id"))
+
+    def filter_query(self, db_map, query):
+        highlighted_object_qry = db_map.query(db_map.relationship_sq).filter(
+            db_map.relationship_sq.c.dimension == self.query_parents("highlight_dimension")
+        )
+        conditions = (
+            and_(db_map.wide_relationship_sq.c.id == x.id, db_map.object_sq.c.id == x.object_id)
+            for x in highlighted_object_qry
+        )
+        return query.filter(or_(*conditions))
+
+    @staticmethod
+    def is_buddy(parent):
+        return isinstance(parent, RelationshipClassObjectHighlightingMapping)
 
 
 class RelationshipObjectMapping(ExportMapping):
@@ -821,6 +945,7 @@ class RelationshipObjectMapping(ExportMapping):
     """
 
     MAP_TYPE = "RelationshipObject"
+    _cached_dimension = None
 
     @staticmethod
     def name_field():
@@ -832,14 +957,17 @@ class RelationshipObjectMapping(ExportMapping):
 
     def _data(self, db_row):
         data = super()._data(db_row).split(",")
-        index = self.index()
+        if self._cached_dimension is None:
+            self._cached_dimension = self.query_parents("dimension")
         try:
-            return data[index]
+            return data[self._cached_dimension]
         except IndexError:
             return ""
 
-    def index(self):
-        return self.parent.index() + 1
+    def query_parents(self, what):
+        if what != "dimension":
+            return super().query_parents(what)
+        return self.parent.query_parents(what) + 1
 
     @staticmethod
     def is_buddy(parent):
@@ -861,18 +989,18 @@ class ParameterDefinitionMapping(ExportMapping):
         )
 
     def filter_query(self, db_map, query):
-        if "object_class_id" in {c["name"] for c in query.column_descriptions}:
+        column_names = {c["name"] for c in query.column_descriptions}
+        if "object_class_id" in column_names:
             return query.outerjoin(
                 db_map.parameter_definition_sq,
                 db_map.parameter_definition_sq.c.object_class_id == db_map.object_class_sq.c.id,
             )
-        if "relationship_class_id" in {c["name"] for c in query.column_descriptions}:
+        if "relationship_class_id" in column_names:
             return query.outerjoin(
                 db_map.parameter_definition_sq,
                 db_map.parameter_definition_sq.c.relationship_class_id == db_map.wide_relationship_class_sq.c.id,
             )
-        # We should never end up here
-        return query
+        raise RuntimeError("Logic error: this code should be unreachable.")
 
     @staticmethod
     def name_field():
@@ -1044,24 +1172,22 @@ class ParameterValueMapping(ExportMapping):
     def filter_query(self, db_map, query):
         if not self._selects_value:
             return query
-        if "object_id" in {c["name"] for c in query.column_descriptions}:
-            return query.outerjoin(
-                db_map.parameter_value_sq,
+        column_names = {c["name"] for c in query.column_descriptions}
+        if "object_id" in column_names:
+            return query.filter(
                 and_(
                     db_map.parameter_value_sq.c.object_id == db_map.object_sq.c.id,
                     db_map.parameter_value_sq.c.parameter_definition_id == db_map.parameter_definition_sq.c.id,
-                ),
+                )
             )
-        if "relationship_id" in {c["name"] for c in query.column_descriptions}:
-            return query.outerjoin(
-                db_map.parameter_value_sq,
+        if "relationship_id" in column_names:
+            return query.filter(
                 and_(
                     db_map.parameter_value_sq.c.relationship_id == db_map.wide_relationship_sq.c.id,
                     db_map.parameter_value_sq.c.parameter_definition_id == db_map.parameter_definition_sq.c.id,
-                ),
+                )
             )
-        # We should never end up here
-        return query
+        raise RuntimeError("Logic error: this code should be unreachable.")
 
     @staticmethod
     def name_field():
@@ -1181,22 +1307,17 @@ class ParameterValueListMapping(ExportMapping):
     MAP_TYPE = "ParameterValueList"
 
     def add_query_columns(self, db_map, query):
-        if self.parent is None:
-            return query.add_columns(
-                db_map.parameter_value_list_sq.c.id.label("parameter_value_list_id"),
-                db_map.parameter_value_list_sq.c.name.label("parameter_value_list_name"),
-            )
         return query.add_columns(
-            db_map.wide_parameter_value_list_sq.c.id.label("parameter_value_list_id"),
-            db_map.wide_parameter_value_list_sq.c.name.label("parameter_value_list_name"),
+            db_map.parameter_value_list_sq.c.id.label("parameter_value_list_id"),
+            db_map.parameter_value_list_sq.c.name.label("parameter_value_list_name"),
         )
 
     def filter_query(self, db_map, query):
         if self.parent is None:
             return query
         return query.outerjoin(
-            db_map.wide_parameter_value_list_sq,
-            db_map.wide_parameter_value_list_sq.c.id == db_map.parameter_definition_sq.c.parameter_value_list_id,
+            db_map.parameter_value_list_sq,
+            db_map.parameter_value_list_sq.c.id == db_map.parameter_definition_sq.c.parameter_value_list_id,
         )
 
     @staticmethod
@@ -1218,19 +1339,21 @@ class ParameterValueListValueMapping(ExportMapping):
     MAP_TYPE = "ParameterValueListValue"
 
     def add_query_columns(self, db_map, query):
-        return query.add_columns(db_map.parameter_value_list_sq.c.value)
+        return query.add_columns(db_map.ord_list_value_sq.c.value, db_map.ord_list_value_sq.c.type)
+
+    def filter_query(self, db_map, query):
+        return query.filter(db_map.ord_list_value_sq.c.parameter_value_list_id == db_map.parameter_value_list_sq.c.id)
 
     @staticmethod
     def name_field():
-        return "value"
+        return None
 
     @staticmethod
     def id_field():
-        return "value"
+        return None
 
     def _data(self, db_row):
-        data = super()._data(db_row)
-        return from_database_to_single_value(data, None)
+        return from_database_to_single_value(db_row.value, db_row.type)
 
     @staticmethod
     def is_buddy(parent):
@@ -1640,7 +1763,7 @@ class _FilteredQuery:
 
     def first(self):
         first = self._query.first()
-        if self._condition(first):
+        if first is not None and self._condition(first):
             return first
         return None
 
@@ -1705,7 +1828,7 @@ def from_dict(serialized):
     Deserializes mappings.
 
     Args:
-        serialized (list): serialize mappings
+        serialized (list): serialized mappings
 
     Returns:
         ExportMapping: root mapping
@@ -1735,9 +1858,11 @@ def from_dict(serialized):
             ParameterValueListValueMapping,
             ParameterValueMapping,
             ParameterValueTypeMapping,
-            RelationshipMapping,
             RelationshipClassMapping,
             RelationshipClassObjectClassMapping,
+            RelationshipClassObjectHighlightingMapping,
+            RelationshipMapping,
+            RelationshipObjectHighlightingMapping,
             RelationshipObjectMapping,
             ScenarioActiveFlagMapping,
             ScenarioAlternativeMapping,
@@ -1760,10 +1885,33 @@ def from_dict(serialized):
         if isinstance(position, str):
             position = Position(position)
         ignorable = mapping_dict.get("ignorable", False)
-        flattened.append(mappings[mapping_dict["map_type"]].reconstruct(position, ignorable, mapping_dict))
+        value = mapping_dict.get("value")
+        header = mapping_dict.get("header", "")
+        filter_re = mapping_dict.get("filter_re", "")
+        flattened.append(
+            mappings[mapping_dict["map_type"]].reconstruct(position, value, header, filter_re, ignorable, mapping_dict)
+        )
     return unflatten(flattened)
 
 
+def legacy_group_fn_from_dict(serialized):
+    """Restores legacy group_fn attribute from serialized mappings.
+
+    group_fn has been removed from export mappings but this serves for backwards compatibility.
+
+    Args:
+        serialized (list): serialized mappings
+
+    Returns:
+        str: name of the first group_fn attribute that was found in the serialized mappings or NoGroup if not found
+    """
+    for mapping_dict in serialized:
+        group_fn = mapping_dict.get("group_fn")
+        if group_fn is not None:
+            return group_fn
+    return NoGroup.NAME
+
+
 def _expand_indexed_data(data, mapping):
     """Expands indexed data and updates the current_leaf attribute.
 
